% This example shows how to do control chart

addpath('./Stochastic_PWGP');    addpath('./Asymptotics')

%% Generate simulation data
%%%% initial setting
% modify the following initalial settings for your purposes
l = 4;              % number of levels
N = 25;             % number of design sites
p = 1;              % dimension of input space X
domain = 10;        % the domain of input space starting from 0
numTest = 0;
% set true values for parameters
tbeta = [0 0 0 0];
tsigma = [2 4 3 5];
tphi= [2 2 2 2];
ttheta = [2.5681    0.5548    2.1481    0.5355    2.1763    1.1510];
tTao = trilelm(PD(ttheta)*PD(ttheta)', -1)';
trndErr = 0.1;
% design sites for Multivator
level = struct('S', {[]}, 'Y', {[]});
design0 = repmat(level, l, 1);
trial = repmat(level, l, 1);
X = linspace(0, domain, N)';
for i = 1 : l
    design0(i).S = X;
    trial(i).S = [];
end
%%%% sample data for parameter Estimation
[design, ~] = randomSP(design0, trial, tbeta, tsigma, tphi, ttheta, trndErr, 5000);

%% Parameter estimation
% modeling the data using PWGP
initTheta = pi/2 * ones(1, l*(l-1)/2);      %initial theta for search algorithm
initPhi = 2*ones(1, l*p);                   %initial phi for search algorithm
initSigma = 20*ones(1,l);
options = optimset('Algorithm','active-set', 'RelLineSrchBnd', 0.5, ...
    'RelLineSrchBndDuration', 500, 'Display','off');
theta0 = [  initPhi     initTheta   initSigma];
UB = [ 10*ones(1,l*p)           pi*ones(1, l*(l-1)/2)-0.1   100*ones(1,l) ];
LB = [ 0*ones(1,l*p)+0.0001     zeros(1, l*(l-1)/2)+0.1     10*ones(1,l)  ];
[dmodelPW, perfPW] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options); 


%% Get asymptotic distribution
[acov1, acov2] = asymp_cov_PWGP(design0, @regpoly0, dmodelPW.phi, dmodelPW.sigma, ...
    dmodelPW.theta(l*p+(1:l*(l-1)/2)), dmodelPW.rndErr);

%% Control Chart
r = 10;     iter = 1000;
acov = acov1(l*p+1:2*l*p,l*p+1:2*l*p)/r;
phi = zeros(iter, l*p);
parfor k = 1 : iter
    disp(k);
    givenPhi = tphi;
    if k>iter/2
        givenPhi(1) = 1.6;  givenPhi(2) = 2.2;
    end
    [design, ~] = randomSP(design0, trial, tbeta, tsigma, givenPhi, ttheta, trndErr, r);
    
    % multivator model fitting
    dmodel = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);
    phi(k,:) = dmodel.phi';
end
figure; plot(diag((phi-repmat(dmodelPW.phi', iter, 1))*inv(acov)*(phi-repmat(dmodelPW.phi', iter, 1))'));
hold on; plot(chi2inv(0.99,size(acov,1))*ones(1,iter))
