function [ r, CC] = corr_MV_gauss( theta, d, pid)
% this is covariance function for MGP
% theta: is the parafor the MGP, include roughness paremeters and
% cross-correlatoin parameters
% d: the distance vector for inputs (design.S)
% pid: the indices of which reponses the intpus (design.S) belongs to
    
    [~, p] = size(d);  % number of differences and dimension of input design sites
    numLevels = max(pid(:, 1));
    k = numLevels;
    % check number of parameters in theta
    numPara = k*p + (k-1)*k/2;
    if length(theta) ~= numPara
        error('incorrect number of parameters')
    end
    
    phi = reshape(theta(1:k*p), p, k)';
   
    % phi i and j
    phiI = phi(pid(:,1),:);
    phiJ = phi(pid(:,2),:); 
    phiIJ = 2*phiI.*phiJ ./ (phiI + phiJ);
    normalizedC = prod((phiI + phiJ)/2 .* (1./phiI + 1./phiJ)/2, 2).^(1/4);
    td = - d.^2 .* phiIJ;
    
    % cross-correlation rate between levels
    t = theta(k*p + 1 : end);
    LL = HD(t);
    CL = LL(pid(:,1),:);
    CL = CL .*  LL(pid(:,2),:);
    CL = sum(CL, 2);   
   
    % cross correlation matrix
    CC = LL*LL';
    
    % correlation
    r = CL .* exp(sum(td,2));
    r = r ./ normalizedC;

function L = HD(T)
    % matrix construction based on hypersphere decomposition
    % T:  parameters 
    if isempty(T)
        L = 1;
        return;
    end
    J = ceil(sqrt(2*length(T)));  % number of levels
    % if J*(J-1)/2 ~= length(T)  error('Incorrect number of parameters!'), end
    L = zeros(J,J); % initialize L matrix

    L(1,1) = 1;
    for i = 2 : J  % row index in L
        id = (i-1)*(i-2)/2 + 1; % starting parameter for row i
        L(i,1) = 1;
        for j = 2 : i
            L(i,j) = L(i,j-1)*sin(T(id+j-2));
        end
        L(i,1:i-1) = (T(id:id+i-2) ~= pi/2) .* cos(T(id:id+i-2)) .* L(i,1:i-1);
    end
    
    