function [ r, CC] = corr_PW_gauss( theta, d, pid, T, Li, Lj)
% this is covariance function for PWGP
% theta: is the parafor the MGP, include roughness paremeters and
% cross-correlatoin parameters
% d: the distance vector for inputs (design.S)
% pid: the indices of which reponses the intpus (design.S) belongs to  

    [~, p] = size(d);  % number of differences and dimension of input design sites
    numLevels = max(pid(:, 1));
    k = numLevels;
    % check number of parameters in theta
    numPara = k*p + (k-1)*k/2;
    if length(theta) ~= numPara
        error('incorrect number of parameters')
    end
    
    phi = reshape(theta(1:k*p), p, k)';
   
    % phi i and j
    phiI = phi(pid(:,1),:);
    phiJ = phi(pid(:,2),:); 
    phiIJ = 2*(1./phiI + 1./phiJ).^(-1);
    td = - d.^2 .* phiIJ;
    
    % cross-correlation rate between levels
    t = theta(k*p + 1 : end);
    T((Li-1)*(Li-2)/2+Lj) = t;
    LL = PD(T, Li, Lj);
    CL = sum(LL(pid(:,1),:) .* LL(pid(:,2),:), 2);   
   
    % cross correlation matrix
    CC = LL*LL';
    
    % correlation
    r = CL .* exp(sum(td,2)) ./ prod((phiI + phiJ)/2 .* (1./phiI + 1./phiJ)/2, 2).^(1/4);
    
   
function L = PD(T, Li, Lj)
    % matrix construction based on hypersphere decomposition
    % T:  parameters 
    if isempty(T)
        L = 1;
        return;
    end
    
    J = ceil(sqrt(2*length(T)));  % number of levels
    % if J*(J-1)/2 ~= length(T)  error('Incorrect number of parameters!'), end
    L = zeros(J,J); % initialize L matrix

    L(1,1) = 1;
    L(2,1) = cos(T(1));
    L(2,2) = sin(T(1));

    for i = 3 : J  % row index in L
        id = (i-1)*(i-2)/2 + 1; % starting parameter for row i
        L(i,1) = cos(T(id));
        for j = 2 : i-1
            k = 1;
            L(i,j) = sin(T(id));
            while k < j-1  L(i,j) = L(i,j)*sin(T(id+k));k = k + 1;end
            L(i,j) = L(i,j)*cos(T(id+j-1));
        end
        L(i,i) = L(i,i-1)/cos(T(id+i-2))*sin(T(id+i-2));
    end
    L((abs(L) < 1e-16)) = 0;
    C = L*L';
    L = tril(ones(2));
    L(2,1) = C(Li, Lj);
    L(2,2) = sqrt(1 - C(Li, Lj)^2);
    
    