function [ design, trial ] = randomSP( design, trial, beta, sigma, phi, crosspara, rndErr, m)
% this function aims to sample data for a MGP given the parameters beta,
% sigma, phi, etc.

    k = length(design);   %number of levels
    sigma = sigma(:);
    roughness_para = phi(:)';
    crosscorr_para = crosspara;
    
    MU = [];
    S = [];     % design sites
    indexLevels = [];
    for i = 1 : k
        S = [S; design(i).S ; trial(i).S];
        indexLevels = [indexLevels; i*ones(size(design(i).S,1)+size(trial(i).S,1), 1)];
        MU = [MU ; beta(i)*ones(size(design(i).S,1)+size(trial(i).S,1), 1)];
    end
    indexLP = indexLevels;   % index of the levels where a point is
    
    % Calculate distances between points
    indexI = (1 : size(S,1))';
    indexJ = (1 : size(S,1))';
    [indexI, indexJ] = meshgrid(indexI, indexJ);
    [indexLevelsI, indexLevelsJ] = meshgrid(indexLevels,indexLevels);
    index = true(size(indexI));
    indexI = indexI(index);
    indexJ = indexJ(index);
    indexLevelsI = indexLevelsI(index);
    indexLevelsJ = indexLevelsJ(index);
    D = S(indexI,:) - S(indexJ,:);    % distances between points    
    indexIJ = [indexI indexJ];      % index matrix
    % matrix storing index of levels
    indexLev = [indexLevelsI indexLevelsJ];
    
    % Set up  R
    n = size(S, 1);
    phi = [roughness_para crosscorr_para];
    [r, CC] = corr_MV_gauss(phi(:)', D, indexLev);
    r = r.*sigma(indexLev(:,1)).*sigma(indexLev(:,2));
    
    idx = find(r);   o = (1 : n)';   
    R = sparse([indexIJ(idx,1); o], [indexIJ(idx,2); o], ...
      [r(idx); rndErr^2*ones(n,1)]);
    SIGMA = full(R);
    
    Y = mvnrnd(MU,SIGMA,m)';
    index = (1:n)';
    index = reshape(index, n/k, k);
    for i = 1 : k
        indexi = index(1:size(design(i).S,1), i);
        design(i).Y = Y(indexi, :);
        indexi = index(1+size(design(i).S,1):end, i);
        trial(i).Y = Y(indexi, :);
    end
    
end

