% this function aims to do the phase1 analysis for the section "cases
% study" in the paper

addpath('..\Stochastic_PWGP\');
addpath('..\Simulation_Study\Asymptotics\Asymptotics_PWGP_Fomular\');

% sample data using from original data
[design, designs] = sample_data();

% assemble data
l = length(design);
p = size(design(1).S,2);
limit = 0.99;

% init
initTheta = pi/2 * ones(1, l*(l-1)/2);      %initial theta for search algorithm
initPhi = 0.1*ones(1,l*p);
initSigma = 7*ones(1,l);
[d1, d2] = meshgrid(1:l, 1:l);
UB = [ 10*repmat(ones(1, p),1,l)            pi*ones(1, l*(l-1)/2)-0.2       20*ones(1,l) ];
LB = [ 0*repmat(ones(1, p),1,l)+0.0001      invPD(exp(-0.03*abs(d1-d2)))    2*ones(1,l)];
clear d1 d2;

% first round asymptotics
options = optimset('Algorithm','active-set', 'RelLineSrchBnd', 0.2, ...
    'RelLineSrchBndDuration', 200, 'Display','off');
theta0 = [  initPhi     initTheta   initSigma];
design = comb_designs(designs);
[dmodel, ~] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);

% theoretical asymptotics
[acov1, acov2 ] = asympt( dmodel );


% first round control chart
acov = acov2(1:l*p,1:l*p);
iter = length(designs);
flag = true(iter ,1);
Q1 = zeros(iter, 1);
parfor k = 1 : iter
    design = designs{k};
    theta0 = [  initPhi     initTheta   initSigma];
    [dmodelPW, ~] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);
    % only consider beta
    estimates = dmodelPW.beta - dmodel.beta;
    
    % Chi Square test
    r = size(design(1).Y,2);
    C = chol(acov/r);
    para = estimates; 
    para = C' \ para;
    Q1(k) = para'*para;
    if Q1(k) > chi2inv(limit,size(acov,1))
        flag(k) = false;
    end
end
save('1st_round.mat');
figure; plot(1:length(Q1), Q1);
hold on; plot(1:length(Q1), chi2inv(limit,size(acov,1))*ones(1,length(Q1))); hold off;




% second round asymptotics
designs = designs(flag);
design = comb_designs(designs);
[dmodel, ~] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);

% theoretical asymptotics
[acov1, acov2 ] = asympt( dmodel );

% second round control chart
acov = acov2(1:l*p,1:l*p);
iter = length(designs);
flag = true(iter ,1);
Q2 = zeros(iter, 1);
parfor k = 1 : iter
    design = designs{k};
    theta0 = [  initPhi     initTheta   initSigma];
    [dmodelPW, ~] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);
    % only consider beta
    estimates = dmodelPW.beta - dmodel.beta;
    
    % Chi Square test
    r = size(design(1).Y,2);
    C = chol(acov/r);
    para = estimates; 
    para = C' \ para;
    Q2(k) = para'*para;
    if Q2(k) > chi2inv(limit,size(acov,1))
        flag(k) = false;
    end
end
save('2st_round.mat');
figure; plot(1:length(Q2), Q2);
hold on; plot(1:length(Q2), chi2inv(limit,size(acov,1))*ones(1,length(Q2))); hold off;