function [ J, H, Cf ] = asymp_cov( design, regr, tphi, tsigma, ttheta, trndErr, method )
    
    k = length(design);
    code = 'syms';
    for i = 1 : k
         code = [code ' ph' num2str(i) ' s' num2str(i)];
    end
    for i = 1 : k
       for j = 1 : i-1
            code = [code ' a' num2str(i) num2str(j)];
       end
    end
    eval(code);
    syms e ;
    sigma = [];
    alpha = [];
    phi = [];
    code = 'alpha = [';
    for i = 1 : k
       for j = 1 : i-1
            code = [code ' a' num2str(i) num2str(j)];
       end
    end
    code = [code ' ];'];
    eval(code);
    
    code = 'phi = [';
    for i = 1 : k
         code = [code ' ph' num2str(i) ';'];
    end
    code = [code ' ];'];
    eval(code);
    
    code = 'sigma = [';
    for i = 1 : k
         code = [code ' s' num2str(i) ';'];
    end
    code = [code ' ];'];
    eval(code);
    
    % chose a method
    if method == 1
        T = PD(ttheta)*PD(ttheta)';
        for i = 1 : k
           for j = 1 : i-1
                p = (i - 2)*(i - 1)/2 + j;
                ttheta(p) = T(i,j);
           end
        end
        T = cross_corr(k, alpha);
    else
        T = cross_corr_HD(k, alpha);
    end
    
    rndErr = e;
    para = [transpose(phi) transpose(sigma) alpha rndErr];
    value = [tphi tsigma ttheta trndErr];
    
    for i = 1 : k
        design(i).F = feval(regr, design(i).S);
    end
    F = blkdiag(design(:).F);
    sCf = sym_cov_full(design, phi, sigma, rndErr, T);
    Cf = eval_cov(sCf, para, value);
    sCf = symcell2mat(sCf);
    Cf = cell2mat(Cf);
    Je = asymp_corr(sCf, Cf, phi, sigma, rndErr, alpha, value);
    Jbeta = F'*inv(Cf)*F;
    I = blkdiag(Jbeta, Je);
    J = I;  H = I;    
end

function aC = asymp_corr(sCf, Cf, phi, sigma, rndErr, alpha, value)
    para = [transpose(phi) transpose(sigma) alpha, rndErr];
    n = length(para);
    for i =1 : n
        eval([char(para(i)) '=', num2str(value(i)) ';']);
    end
    aC = zeros(n, n);
    dCs = zeros([size(Cf),n]);
    for i = 1 : n
        dC = eval(diff(sCf, para(i)));
        dCs(:,:,i) = dC;
    end
    for i = 1 : n
        for j = 1 : i
            disp([i,j])
            aC(i,j) = 1/2*trace(inv(Cf)*dCs(:,:,i)*inv(Cf)*dCs(:,:,j));
            aC(j,i) = aC(i,j);
        end
    end
end


function C = sym_cov_full(design, phi, sigma, rndErr, T)
    
    l = length(design);
    C = cell(l,l);
    
    for i = 1:l
        for j = 1:l
            C{i,j} = sym_cov_blk(design, phi, sigma, rndErr, T, i, j);
        end
    end
    
end

function C = eval_cov(sC, para, value)
    C = cell(size(sC));
    n = length(para);
    % to speed up by puting all symbolic variables in to memory instead of
    % using the function "subs"
    for i =1 : n
        eval([char(para(i)) '=', num2str(value(i)) ';']);
    end
    for i = 1 : size(sC,1)
        for j = 1 : size(sC,2)
            C{i,j} = eval(sC{i,j});
        end
    end
end

function C = symcell2mat(sC)
    indi = 0;
    for i = 1 : size(sC,1)
        indj = 0;
        for j = 1 : size(sC,2)
            Cij = sC{i,j};
            [s1, s2] = size(Cij);
            C(indi+(1:s1), indj+(1:s2)) = Cij;
            indj = indj + s2;
        end
        indi = indi + s1;
    end 
end

function C = sym_cov_blk(design, phi, sigma, rndErr, T, i, j)
    normalizedC = ((phi(i)+phi(j))/2*(1/phi(i)+1/phi(j))/2)^(1/4);
    phiIJ = 2*phi(i)*phi(j) / (phi(i)+phi(j));
    [S2, S1] = meshgrid(design(j).S, design(i).S); 
    C = exp(-(S1-S2).^2 * phiIJ) / normalizedC * sigma(i)* sigma(j)*T(i,j);
    if i == j
        C = C + eye(size(C))*rndErr^2;
    end
end


function T = cross_corr(k, alpha)
    
    T = sym(ones(k));
    for i = 2 : k
        for j = 1 : i-1
            p = (i - 2)*(i - 1)/2 + j;
            T(i,j) = alpha(p);
            T(j,i) = alpha(p);
        end
    end
end

function T = cross_corr_HD(k, alpha)
    
    L = sym(zeros(k));
    L(1,1) = sym(1);
    for i = 2 : k
        tL = 1;
        for j = 1 : i-1
            p = (i - 2)*(i - 1)/2 + j;
            L(i,j) = cos(alpha(p))*tL;
            tL = tL*sin(alpha(p));    
        end
        L(i,i) = tL;
    end
    T = L*transpose(L);
end
