% this function is to implement the example in section 4.2 for comparing
% separable and nonseparable covariance functions

addpath('..\..\Stochastic_PWGP\')
initSetting;

figure; 

iter = 1000;
r = 10;
tph = tphi;

initTheta = pi/2 * ones(1, l*(l-1)/2);      %initial theta for search algorithm
initPhi = 2*ones(1, l*p);                   %initial phi for search algorithm
initSigma = 30*ones(1,l);
options = optimset('Algorithm','active-set', 'RelLineSrchBnd', 0.5, ...
    'RelLineSrchBndDuration', 500, 'Display','off');

phi = zeros(iter, l*p);
theta = zeros(iter, l*(l-1)/2);
beta = zeros(iter, l);
sigma = zeros(iter, l);
Tao = zeros(iter, l*(l-1)/2);
rndErr = zeros(iter, 1);

% multivator model fitting
theta0 = [  initPhi     initTheta   initSigma];
UB = [ 3*ones(1,l*p)            pi*ones(1, l*(l-1)/2)-0.2   200*ones(1,l) ];
LB = [ 1*ones(1,l*p)            zeros(1, l*(l-1)/2)+0.2     5*ones(1,l)  ];

parfor k = 1 : iter
    disp(k);
    tphi = tph;
    if k>iter/2
        tphi(1) = 1.8;  tphi(2) = 2.1;
    end
    [design, ~] = randomSP(design0, trial, tbeta', tsigma',tphi, ttheta, trndErr, r);

    [dmodelPW, perfPW] = dacefit_Pairwise(design, @regpoly0, {@corr_PW_gauss @corr_MV_gauss}, theta0, LB, UB, 'fmincon', options);
    phi(k,:) = dmodelPW.phi';
    beta(k,:) = dmodelPW.beta';
    sigma(k,:) = dmodelPW.sigma';
    theta(k,:) = dmodelPW.omega';
    Tao(k,:) = trilelm(dmodelPW.CC, -1)';
    rndErr(k,:) = dmodelPW.rndErr;
end
load('.\Asymptotics_PWGP_Fomular\theoretical_asymptotics_N=25_sep.mat','acov1');
acov = acov1(5:l*p+4,5:l*p+4)/r;
tphi = tph;
subplot(2,1,2); hold on;
plot(diag((phi-repmat(tphi, iter, 1))*inv(acov)*(phi-repmat(tphi, iter, 1))'));
plot(chi2inv(0.99,size(acov,1))*ones(1,iter))
hold off;
axis([0 1000 0 100]);
save('NonSep.mat');




% store parameter estimates 
phi = zeros(iter, p);
theta = zeros(iter, l*(l-1)/2);
beta = zeros(iter, l);
sigma = zeros(iter, l);
Tao = zeros(iter, l*(l-1)/2);
rndErr = zeros(iter, 1);

% setting for searching algorithm for MLE
initTheta = pi/2 * ones(1, l*(l-1)/2);      
initPhi = 2;                                
initSigma = 20*ones(1,l);                   
options = optimset('Algorithm','active-set', 'RelLineSrchBnd', 0.5, ...
    'RelLineSrchBndDuration', 500, 'Display','off');

theta0 = [  initPhi     initTheta   initSigma];
UB = [ 10*ones(1,p)           pi*ones(1, l*(l-1)/2)-0.1   200*ones(1,l) ];
LB = [ 0*ones(1,p)+0.0001     zeros(1, l*(l-1)/2)+0.1     5*ones(1,l)  ];
parfor k = 1 : iter
    disp(k);
    tphi = tph;
    if k>iter/2
        tphi(1) = 1.8;  tphi(2) = 2.1;
    end
    [design, ~] = randomSP(design0, trial, tbeta', tsigma',tphi, ttheta, trndErr, r);
    
    % multivator model fitting
    [dmodelMV, perfMV] = dacefit_MV_Sep(design, @regpoly0, @corr_MV_gauss, theta0, LB, UB, 'fmincon', options);
    phi(k,:) = dmodelMV.phi';
    beta(k,:) = dmodelMV.beta';
    sigma(k,:) = dmodelMV.sigma';
    theta(k,:) = dmodelMV.omega';
    Tao(k,:) = trilelm(dmodelMV.CC, -1)';
    rndErr(k,:) = dmodelMV.rndErr;
end

load('.\Asymptotics_MGP_Sep_Fomular\theoretical_asymptotics_N=25.mat','acov1')
acov = acov1(5:p+4,5:p+4)/r;
tphi = tph(1);
subplot(2,1,1); plot(phi);
hold on;   plot(repmat(norminv([0.005 0.995],tphi,sqrt(acov))', 1, iter)');
hold off;
axis([0 1000 1.7 2.3]);
save('Sep.mat');